package gov.va.med.ccht.controller;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.springframework.batch.core.BatchStatus;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.scheduling.quartz.CronTriggerBean;
import org.springframework.scheduling.quartz.JobDetailBean;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;

import gov.va.med.ccht.model.Schedule;
import gov.va.med.ccht.model.report.ReportSchedule;
import gov.va.med.ccht.model.terminology.DayOfWeek;
import gov.va.med.ccht.model.terminology.Month;
import gov.va.med.ccht.model.terminology.RunFrequency;
import gov.va.med.ccht.model.terminology.WeekOfMonth;
import gov.va.med.ccht.service.common.BatchProcessTriggerGroups;
import gov.va.med.ccht.ui.model.BatchProcessDetailForm;
import gov.va.med.ccht.ui.model.FormattedDateForm;
import gov.va.med.ccht.ui.model.ReportScheduleForm;
import gov.va.med.fw.batchprocess.BatchProcessDetail;
import gov.va.med.fw.model.batchprocess.BatchJobExecution;
import gov.va.med.fw.model.batchprocess.JobConfig;
import gov.va.med.fw.scheduling.TriggerStatus;
import gov.va.med.fw.security.Permission;
import gov.va.med.fw.security.UserPrincipal;
import gov.va.med.fw.service.ConfigurationConstants;
import gov.va.med.fw.service.jms.JMSPayload;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.DateUtils;
import gov.va.med.fw.util.StringUtils;
import gov.va.med.fw.util.date.DateWithTimeZone;
import gov.va.med.fw.util.date.TimeZoneUtils;

@Controller
@SessionAttributes(value = { "command" })
public class ManageScheduledJobsController extends CommonCchtController implements ApplicationContextAware {

	public static final String JOB_NAME = "jobName";
	public static final String JOB_PARAMETERS = "jobParameters";
	public static final String USER_ID = "userId";
	public static final int MAX_USER_ID_LENGTH = 30;

	private ApplicationContext applicationContext;

	// move to property if needed later?
	@Value("runBatchJobJMSPayload")
	private String runBatchJobJMSPayloadName;

	@RequestMapping("/scheduledJobs")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody Map<String, Object> getScheduledJobs() throws Exception {
		Map<String, Object> results = new HashMap<>();
		UserPrincipal user = getCurrentUser();
		List<BatchProcessDetailForm> jobAndTriggerStatus = new ArrayList<BatchProcessDetailForm>();

		@SuppressWarnings("unchecked")
		Collection<BatchProcessDetail> batchProcessDetails = (Collection<BatchProcessDetail>) batchProcessInvoker
				.getBatchProcessDetails();
		for (BatchProcessDetail detail : batchProcessDetails) {
			BatchProcessDetailForm form = new BatchProcessDetailForm();
			form.setId(detail.getCode());
			boolean canExecuteOrView = true;
			// set batch process details name, group ..
			form.setJobName(detail.getJobName());
			form.setGroupName(detail.getGroupName());
			form.setJobDescription(detail.getJobDescription());
			form.setTriggerName(detail.getTriggerName());
			form.setTriggerGroup(detail.getTriggerGroup());

			// set trigger status and next fire time
			if (StringUtils.isNotEmpty(detail.getTriggerName())) {
				TriggerStatus triggerStatus = schedulingService.getTriggerStatus(detail.getTriggerName(),
						detail.getTriggerGroup());
				if (triggerStatus != null) {
					form.setStatus(triggerStatus.getStatus());
					form.setNextFireTime(new FormattedDateForm(triggerStatus.getNextFireTimeWithTimeZone()));
				}
			}

			JobConfig jobConfig = batchProcessService.getJobConfig(detail.getJobName(), detail.getGroupName());
			if (jobConfig != null) {
				if (jobConfig.getJobSchedule() != null) {
					form.setJobSchedule(jobConfig.getJobSchedule());
					form.setJobScheduleDescription(jobConfig.getJobScheduleDescription());
					if (StringUtils.isNotEmpty(jobConfig.getJobScheduleText())) {
						Schedule schedule = scheduleConversionService.toSchedule(jobConfig.getJobScheduleText());
						ReportScheduleForm rsForm = new ReportScheduleForm();
						reportConversionService.convert(schedule, rsForm);
						form.setReportSchedule(rsForm);
					}
				}
				form.setEmailDistributionList(jobConfig.getEmailDistributionList());
				canExecuteOrView = isAllowedToExecuteOrViewBatchJob(user, jobConfig.getPermissions());
			}
			if (canExecuteOrView) {
				jobAndTriggerStatus.add(form);
			}
		}

		Collections.sort(jobAndTriggerStatus);
		results.put("jobs", jobAndTriggerStatus);
		return results;
	}

	@RequestMapping("/scheduledJobs/active")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody Map<String, Object> getActiveJobs() throws Exception {
		Map<String, Object> results = new HashMap<>();
		@SuppressWarnings("rawtypes")
		List activeJobs = batchProcessService.getJobResults(BatchStatus.STARTED);
		results.put("jobs", activeJobs);
		return results;

	}

	@RequestMapping("/scheduledJobs/configUpdate")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean submitConfigUpdate(@RequestParam String jobName, @RequestParam String groupName,
			@RequestParam String emails, @RequestParam String jobStatus, @RequestParam String previousJobStatus)
			throws Exception {
		JobConfig jobConfig = batchProcessService.getJobConfig(jobName, groupName);
		if (jobConfig == null) {
			jobConfig = new JobConfig();
			jobConfig.setJobName(jobName);
			jobConfig.setJobGroup(groupName);
		}

		jobConfig.setEmailDistributionList(emails);
		batchProcessService.saveJobConfig(jobConfig);

		if (StringUtils.isNotEmpty(jobStatus) && !jobStatus.equals(previousJobStatus)) {
			if (TriggerStatus.NORMAL_STATUS.equals(jobStatus)) {
				schedulingService.resumeJob(jobName, groupName);
			} else if (TriggerStatus.PAUSED_STATUS.equals(jobStatus)) {
				schedulingService.pauseJob(jobName, groupName);
			}
		}

		return true;
	}

	@RequestMapping("/scheduledJobs/execute")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean submitExecuteJob(@RequestParam String jobName) throws Exception {
		JobDetailBean jobDetailsBean = (JobDetailBean) applicationContext.getBean(jobName);
		String jobNameFromBean = (String) jobDetailsBean.getJobDataMap().get(JOB_NAME);

		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put(JOB_NAME, jobNameFromBean);
		map.put(JOB_PARAMETERS, "");
		map.put(USER_ID, getCurrentUser().getUserCredentials().getUserID());

		JMSPayload jmsPayLoad = applicationContext.getBean(runBatchJobJMSPayloadName, JMSPayload.class);
		jmsPayLoad.setPayload(map);

		// set header properties
		Map<String, Serializable> props = new HashMap<String, Serializable>();
		props.put(ConfigurationConstants.DEFAULT_MESSAGE_TYPE, jmsPayLoad.getTargetServiceDescriptor());
		props.put(ConfigurationConstants.DEFAULT_MESSAGE_INITIATER,
				StringUtils.left(jobNameFromBean, MAX_USER_ID_LENGTH));
		props.put(ConfigurationConstants.DEFAULT_MESSAGE_ORIGINATING_TIMEZONE, TimeZoneUtils.getTimeZone().getID());

		batchProcessMessageService.send((Serializable) jmsPayLoad.getPayload(), props);
		return true;
	}

	@RequestMapping("/scheduledJobs/scheduleValidate")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody Map<String, Object> validateJobSchedule(@RequestParam String jobName,
			@RequestParam String runFrequency, @RequestParam String scheduleType, @RequestParam String dayOfMonth,
			@RequestParam String weekOfMonth, @RequestParam String dayOfWeek, @RequestParam String dayOfQuarter,
			@RequestParam String month, @RequestParam Integer numericDayOfMonth,
			@DateTimeFormat(pattern = DateUtils.MMDDYYYY) @RequestParam Date dateToGenerate, @RequestParam Integer hour,
			@RequestParam Integer minute, @RequestParam String daysOfWeekCSV) throws Exception {
		ReportSchedule reportSchedule = new ReportSchedule();

		ReportScheduleForm reportScheduleForm = new ReportScheduleForm();
		reportScheduleForm.setRunFrequency(new TermType("", runFrequency));
		reportScheduleForm.setScheduleType(new TermType("", scheduleType));
		reportScheduleForm.setDayOfMonth(new TermType("", dayOfMonth));
		reportScheduleForm.setWeekOfMonth(new TermType("", weekOfMonth));
		reportScheduleForm.setDayOfWeek(new TermType("", dayOfWeek));
		reportScheduleForm.setDayOfQuarter(new TermType("", dayOfQuarter));
		reportScheduleForm.setMonth(new TermType("", month));
		reportScheduleForm.setNumericDayOfMonth(numericDayOfMonth);

		FormattedDateForm dateToGenerateForm = new FormattedDateForm();
		dateToGenerateForm.setDate(dateToGenerate);
		reportScheduleForm.setDateToGenerate(dateToGenerateForm);

		reportScheduleForm.setHour(hour);
		reportScheduleForm.setMinute(minute);

		List<TermType> daysOfWeek = new ArrayList<>();
		for (String code : daysOfWeekCSV.split(","))
			daysOfWeek.add(new TermType("", code));

		reportScheduleForm.setDaysOfWeek(daysOfWeek);

		reportConversionService.convert(reportScheduleForm, reportSchedule);
		reportScheduleForm.setCronExpression(reportSchedule.getCronExpression());
		reportScheduleForm.setFriendlyDescription(reportSchedule.getFriendlyDescription());
		if (reportSchedule.getNextScheduledDate() != null) {
			reportScheduleForm.setNextFireTime(new FormattedDateForm(
					new DateWithTimeZone(reportSchedule.getNextScheduledDate(), TimeZoneUtils.getTimeZone())));
		} else {
			reportScheduleForm.setNextFireTime(null);
		}

		Map<String, Object> results = new HashMap<>();
		results.put("reportSchedule", reportScheduleForm);
		results.put("jobScheduleParamsCSV", scheduleConversionService.toText(reportSchedule));
		return results;
	}

	@RequestMapping("/scheduledJobs/scheduleUpdate")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean submitJobScheduleUpdate(@RequestParam String jobName, @RequestParam String groupName,
			@RequestParam String scheduleJobCronInput, @RequestParam String jobScheduleDescription,
			@RequestParam String scheduleJobScheduleParamsCSV, @RequestParam String triggerName,
			@RequestParam String triggerGroup) throws Exception {
		JobConfig jobConfig = batchProcessService.getJobConfig(jobName, groupName);
		if (jobConfig == null) {
			jobConfig = new JobConfig();
			jobConfig.setJobName(jobName);
			jobConfig.setJobGroup(groupName);
		}
		// New schedule is empty
		if (StringUtils.isEmpty(scheduleJobCronInput)) {
			return true;
		}

		jobConfig.setJobSchedule(scheduleJobCronInput);
		jobConfig.setJobScheduleDescription(jobScheduleDescription);
		// convert the schedule object
		jobConfig.setJobScheduleText(scheduleJobScheduleParamsCSV);
		batchProcessService.saveJobConfig(jobConfig);
		Trigger trigger = schedulingService.getTrigger(triggerName, triggerGroup);

		if (trigger != null && trigger instanceof CronTrigger) {
			((CronTrigger) trigger).setCronExpression(jobConfig.getJobSchedule());
			schedulingService.reschedule(triggerName, triggerGroup, (CronTrigger) trigger);
		} else {

			// Construct the name by configuration standards
			if (StringUtils.isEmpty(triggerName)) {
				triggerName = (jobName.substring(0, 1)).toLowerCase() + jobName.substring(1) + "CronTrigger";
			}
			try {
				CronTriggerBean cronTriggerBean = applicationContext.getBean(triggerName, CronTriggerBean.class);
				if (cronTriggerBean != null) {
					JobDetail jobDetail = cronTriggerBean.getJobDetail();
					cronTriggerBean.setCronExpression(jobConfig.getJobSchedule());
					if (jobDetail != null)
						schedulingService.schedule(jobDetail, cronTriggerBean);
					else
						schedulingService.schedule(cronTriggerBean);
				}
			} catch (Exception e) {
				throw new Exception(
						"Missing or invalid configuration, could not schedule the batch job for " + jobName);
			}
		}

		return true;
	}

	@RequestMapping("/scheduledJobs/resumeAll")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean submitResumeAll() throws SchedulerException {
		schedulingService.resumeTriggerGroup(BatchProcessTriggerGroups.STATIC_BATCH_JOBS);
		return true;
	}

	@RequestMapping("/scheduledJobs/pauseAll")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean submitPauseAll() throws SchedulerException {
		schedulingService.pauseTriggerGroup(BatchProcessTriggerGroups.STATIC_BATCH_JOBS);
		return true;
	}

	@RequestMapping(value = "/manageScheduledJobs.html", method = RequestMethod.GET)
	public String showManageScheduledJobs(Model model) throws Exception {
		model.addAttribute("allJobStatuses", Arrays.asList(TriggerStatus.NORMAL_STATUS, TriggerStatus.PAUSED_STATUS));
		model.addAttribute("allRunFrequencies", terminologyCache.getTerms(RunFrequency.class));
		model.addAttribute("allWeekOfMonthTypes", terminologyCache.getTerms(WeekOfMonth.class));
		model.addAttribute("allDayOfWeekTypes", terminologyCache.getTerms(DayOfWeek.class));
		model.addAttribute("allMonths", terminologyCache.getTerms(Month.class));

		return "manageScheduledJobs";
	}

	private boolean isAllowedToExecuteOrViewBatchJob(UserPrincipal user, Set<Permission> permissions) {
		if (permissions != null && permissions.size() > 0) {
			for (Permission permission : permissions) {
				String code = permission.getName();
				if (user.isPermissionGranted(code)) {
					return true;
				}
			}
		} else {
			return true;
		}
		return false;
	}

	@RequestMapping("/scheduledJobs/jobHistory")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody Map<String, Object> getJobHistory(@RequestParam String jobName, @RequestParam String jobGroup)
			throws Exception {
		Map<String, Object> results = new HashMap<>();

		@SuppressWarnings("unchecked")
		List<BatchJobExecution> jobResults = (List<BatchJobExecution>) batchProcessService
				.getFinishedJobResults(jobName, jobGroup);
		results.put("history", jobResults);

		return results;
	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}

}
