package gov.va.med.ccht.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.naming.SizeLimitExceededException;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.multipart.MultipartFile;

import gov.va.med.ccht.controller.validators.QIRFormValidator;
import gov.va.med.ccht.model.CCHTRoles;
import gov.va.med.ccht.model.User;
import gov.va.med.ccht.model.inventory.DeviceType;
import gov.va.med.ccht.model.inventory.SimpleFacility;
import gov.va.med.ccht.model.inventory.SimpleVisn;
import gov.va.med.ccht.model.inventory.Vendor;
import gov.va.med.ccht.model.qir.QIR;
import gov.va.med.ccht.model.qir.QIRAttachment;
import gov.va.med.ccht.model.qir.QIRSimpleAttachment;
import gov.va.med.ccht.model.qir.QIRType;
import gov.va.med.ccht.model.terminology.DocumentType;
import gov.va.med.ccht.service.common.TerminologyCache;
import gov.va.med.ccht.service.qir.QIRService;
import gov.va.med.ccht.ui.model.QIRForm;
import gov.va.med.fw.model.EntityKeyFactory;
import gov.va.med.fw.model.lookup.Lookup;
import gov.va.med.fw.ui.model.TermType;
import gov.va.med.fw.util.StringUtils;

@Controller
@SessionAttributes(value = { "command" })
public class NewQIRController extends CommonCchtController {
	
	@Autowired
	private InventoryController inventoryController;
	@Autowired
	private QIRConversionService qirConversionService;
	@Autowired
	private QIRFormValidator validator;
	@Value("#{cchtProperties['qir.maxCombinedAttachmentBytes']}")
	private int maxCombinedAttachmentLength;
	
	
    @InitBinder
    public void initBinder(WebDataBinder binder)
    {      	
    	//binder.setDisallowedFields("serialNumber");
    }
    
	@RequestMapping(value = "/newQir.html", method = RequestMethod.GET)
	public String showNewQIRForm(Model model) throws Exception {
		QIRForm qirForm = new QIRForm();

		List<DeviceType> terms = terminologyCache.getTerms(DeviceType.class);
		Map<Long, DeviceType> devices = inventoryController.getDevicesLookUpMap(terms);

		Map<Long, Long> deviceVendorMap = new HashMap<Long, Long>();
		for (Map.Entry<Long, DeviceType> device : devices.entrySet()) {
			Long vendorId = device.getValue().getVendor().getId();
			Long deviceId = device.getValue().getIdentifier();
			deviceVendorMap.put(deviceId, vendorId);
		}
		qirForm.setDevices(devices);
		qirForm.setDeviceVendorMap(deviceVendorMap);

		List<QIRType> qirTypes = terminologyCache.getTerms(QIRType.class);
		Map<String, String> qirTypeMap = new LinkedHashMap<String, String>();
		for (Lookup t : qirTypes) {
			qirTypeMap.put(t.getCode().toString(), t.getName());
		}
		qirForm.setQirTypes(qirTypeMap);

		qirForm.setVendors(inventoryController.getVendorsLookUpMap());

		Map<String, SimpleFacility> userFacilities = null;
		SortedSet<SimpleFacility> list = new TreeSet<>(getUserFacilities());
		userFacilities = new LinkedHashMap<String, SimpleFacility>();
		for (SimpleFacility facility : list) {
			userFacilities.put(facility.getCode(), facility);
		}
		qirForm.setFacilities(userFacilities);

		model.addAttribute("command", qirForm);
		return "newQir";
	}

	@RequestMapping(value = "/newQir.html", params = { "Submit" }, method = RequestMethod.POST)
	public String saveNewQIR(@ModelAttribute("command") @Valid QIRForm command, BindingResult result,
			SessionStatus status, HttpServletRequest request, Model model) throws Exception {
		validator.validate(command, result);
		if (result.hasErrors())
			return "newQir";

		QIR qir = new QIR();

		populateQIRForm(command);
		qirConversionService.convert(command, qir);

		/*
		 * Following two methods should be within a single transaction, but they
		 * want to preserve the original semantics - CPB
		 */
		qirService.updateQIR(qir);
		addAttachments(command, qir, terminologyCache, qirService, request, maxCombinedAttachmentLength);

		userNotifier.notifyUserOnceWithMessage(request, "New QIR created successfully.");
		status.setComplete();
		return "redirect:" + basePath + "newQir.html";
	}

	public static void addAttachments(QIRForm command, QIR qir, TerminologyCache terminologyCache,
			QIRService qirService, HttpServletRequest request, int maxCombinedAttachmentLength) throws SizeLimitExceededException, Exception {
		DocumentType unknownDocType = null;

		List<QIRAttachment> filesToAdd = new ArrayList<>();
		List<QIRAttachment> filesToDelete = new ArrayList<>();

		for (MultipartFile file : command.getAttachmentFiles()) {
			String fileName = file.getOriginalFilename();
			String fileSuffix = fileName == null ? "" : StringUtils.substringAfterLast(fileName.toUpperCase(), ".");

			DocumentType docType = terminologyCache.getTermByName(DocumentType.class, fileSuffix);
			if (docType == null) {
				if (unknownDocType == null)
					unknownDocType = terminologyCache.getTermByCode(DocumentType.class, "10" /* "object" */);
				docType = unknownDocType;
			}

			QIRAttachment qirAttachment = new QIRAttachment(qir.getId(), file.getBytes(), (int) file.getSize(),
					new Date(), docType, fileName);
			filesToAdd.add(qirAttachment);
		}

		long[] deletedAttachments = ServletRequestUtils.getLongParameters(request, "deletedAttachments");
		for (long id : deletedAttachments) {
			QIRAttachment a = qirService.getQIRAttachment(EntityKeyFactory.createEntityKey(id, QIRAttachment.class));
			filesToDelete.add(a);
		}

		int length = 0;
		for (QIRSimpleAttachment a : qir.getAttachments())
			length += a.getLength();
		for (QIRAttachment a : filesToAdd)
			length += a.getLength();
		for (QIRAttachment a : filesToDelete)
			length -= a.getLength();

		if (length > maxCombinedAttachmentLength)
			throw new SizeLimitExceededException("Exceeded max file size of " + (maxCombinedAttachmentLength / 1024.0 / 1024.0) + "Mb.");

		for (QIRAttachment a : filesToAdd)
			qirService.updateQIRAttachment(a);
		for (long id : deletedAttachments)
			qirService.deleteQIRAttachment(id);
	}

	private void populateQIRForm(QIRForm qirForm) {
		User theUser = (User) getCurrentUser();
		SimpleVisn visn = theUser.getVisn();

		if (visn != null) {
			TermType theVisn = new TermType();
			theVisn.setLabel(visn.getDescription());
			theVisn.setValue(visn.getDescription());
			qirForm.setVisn(theVisn);
		}

		qirForm.setQirStatusType(new TermType("New", "1"));
		qirForm.setSubmittedByName(theUser.getUserCredentials().getUserID());
		qirForm.setSubmittedBy(theUser.getUserCredentials().getUserID());
		qirForm.setSubmittedDate(new Date());
		qirForm.setStatusChangeDate(qirForm.getSubmittedDate());

		Long selectedDeviceId = qirForm.getSelectedDeviceId();

		if (selectedDeviceId > 0) {
			DeviceType deviceType = qirForm.getDevices().get(selectedDeviceId);
			TermType device = new TermType();
			device.setValue(deviceType.getCode());
			qirForm.setDeviceType(device);
		}

		Long selectedQirTypeId = qirForm.getSelectedQirType();
		String typeName = qirForm.getQirTypes().get(selectedQirTypeId);
		if (selectedQirTypeId != null && selectedQirTypeId > 0) {
			TermType qirType = new TermType();
			qirType.setLabel(typeName);
			qirType.setValue(selectedQirTypeId.toString());
			qirForm.setQirType(qirType);
		}

		Long selectedVendorId = qirForm.getSelectedVendorId();
		Vendor selectedVendor = qirForm.getVendors().get(selectedVendorId);
		if (selectedVendor != null) {
			TermType vendor = new TermType();
			vendor.setValue(selectedVendor.getCode());
			vendor.setLabel(selectedVendor.getName());
			qirForm.setVendor(vendor);
		}

		String selectedFacilityStationNumber = qirForm.getSelectedFacilityId();
		TermType facility = new TermType();
		facility.setValue(selectedFacilityStationNumber);
		qirForm.setFacility(facility);
	}

	private SortedSet<SimpleFacility> getUserFacilities() throws Exception {
		User currentUser = (User) getCurrentUser();
		SortedSet<SimpleFacility> terms = new TreeSet<SimpleFacility>();
		if (currentUser.isPermissionGranted(CCHTRoles.NATIONAL_ADMIN)
				|| currentUser.isPermissionGranted(CCHTRoles.APPLICATION_ADMIN)
				|| currentUser.isPermissionGranted(CCHTRoles.MANAGEMENT)) {
			terms = inventoryService.getSimpleFacilities();

		} else if (currentUser.isPermissionGranted(CCHTRoles.VISN_ADMIN)
				|| currentUser.isPermissionGranted(CCHTRoles.QIR_ORIGINATOR)) {
			if (currentUser.getVisn() != null) {
				terms = inventoryService.getSimpleFacilities(currentUser.getVisn());
			} else if (currentUser.getFacility() != null) {
				terms.addAll(inventoryDAO.getSimpleFacilities(currentUser.getFacility().getVisn()));
			}
		} else {
			if (currentUser.getFacility() != null) {
				terms.add(currentUser.getFacility());
			}
			if (currentUser.getSecondaryFacility() != null) {
				terms.add(currentUser.getSecondaryFacility());
			}
		}
		return terms;
	}
}
