package gov.va.med.ccht.ui.common;

import java.io.Serializable;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.HibernateOptimisticLockingFailureException;
import org.springframework.stereotype.Controller;

import gov.va.med.ccht.model.CCHTRoles;
import gov.va.med.ccht.model.Schedule;
import gov.va.med.ccht.service.common.ExceptionService;
import gov.va.med.ccht.service.common.RegistrationService;
import gov.va.med.ccht.service.common.SecurityService;
import gov.va.med.ccht.service.common.TerminologyCache;
import gov.va.med.ccht.service.common.TerminologyException;

import gov.va.med.ccht.service.inventory.InventoryService;
import gov.va.med.ccht.service.qir.QIRService;
import gov.va.med.fw.model.lookup.Lookup;
import gov.va.med.fw.model.lookup.SeverityType;
import gov.va.med.fw.security.SecurityContext;
import gov.va.med.fw.security.SecurityContextHelper;
import gov.va.med.fw.security.UserPrincipal;
import gov.va.med.fw.service.AbstractComponent;
import gov.va.med.fw.service.ServiceOptimisticLockException;
import gov.va.med.fw.ui.model.TermType;

/**
 * This is the base class all the UI services should extend. It provides basic
 * facilities needed for all the UI services like access to ServletContext,
 * Logged in user, currently selected Patient and ...
 * 
 * @author Muddaiah Ranga
 */
public abstract class AbstractController extends AbstractComponent implements CCHTRoles {
	
	@Autowired
	protected ExceptionService exceptionService;
	@Autowired
	protected TerminologyCache terminologyCache;
	@Autowired
	protected InventoryService inventoryService;
	@Autowired
	protected RegistrationService registrationService;
	@Autowired
	protected SecurityService securityService;
	@Autowired
	protected QIRService qirService;
//	@Autowired
//	protected DMPService dmpService;

	/**
	 * Gets the HttpRequest object.
	 */
	protected HttpServletRequest getHttpRequest() {
		return FlexUtils.getHttpRequest();
	}

	/**
	 * Gets the UserPrincipal object for the logged in user.
	 */
	protected String getUserPrincipal() {
		// return IHTASession.getUserContext().getName();
		return getCurrentUser().getName();
	}

	public Serializable getSessionData(String key) {
		// return IHTASession.getSessionData(key);
		return (Serializable) getHttpSession().getAttribute(key);
	}

	public void setSessionData(String key, Serializable value) {
		// IHTASession.setSessionData(key, value);
		getHttpSession().setAttribute(key, value);
	}

	public Lookup getByCode(Class<? extends Lookup> terminologyType, String code) throws TerminologyException {
		return terminologyCache.getTermByCode(terminologyType, code);
	}

	public <T extends Lookup> List<T> findAll(Class<T> terminologyType) throws TerminologyException {
		return terminologyCache.getTerms(terminologyType);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.va.med.IHTA.ui.HttpSessionProvider#getHttpSession()
	 */
	public HttpSession getHttpSession() {
		return FlexUtils.getHttpSession();
	}

	/**
	 * @return the IHTASession
	 */
	/*
	 * public IHTASession getIHTASession() { return IHTASession; }
	 */
	/**
	 * @param IHTASession
	 *            the IHTASession to set
	 */
	/*
	 * public void setIHTASession(IHTASession IHTASession) { this.IHTASession =
	 * IHTASession; }
	 */
	protected ControllerException handleException(String message, Throwable exception) {
		return handleException(message, exception, null, SeverityType.ERROR);
	}

	protected ControllerException handleException(Throwable exception, Object producer) {
		return handleException(null, exception, producer);
	}

	protected ControllerException handleException(String message, Throwable exception, Object producer) {
		return handleException(message, exception, producer, SeverityType.ERROR);
	}

	protected ControllerException handleException(String message, Throwable exception, Object producer,
			SeverityType severityType) {
		ControllerException cEx = null;
		if (exception != null) {
			if (exception instanceof HibernateOptimisticLockingFailureException) {
				ServiceOptimisticLockException optEx = new ServiceOptimisticLockException(
						"Optimistic lock error occured");
				optEx.setErrorType(ServiceOptimisticLockException.DATA_CHANGED_ERROR);
				cEx = new ControllerException(ServiceOptimisticLockException.DATA_CHANGED_ERROR, exception,
						ServiceOptimisticLockException.DATA_CHANGED_ERROR);
				cEx.setCallStatus(createControllerResult(optEx));
			} else if (exception instanceof ServiceOptimisticLockException) {
				ServiceOptimisticLockException optEx = (ServiceOptimisticLockException) exception;
				cEx = new ControllerException(optEx.getErrorType(), exception, optEx.getErrorType());
				cEx.setCallStatus(createControllerResult(optEx));
			}
		}
		if (cEx == null) {
			/*
			 * EntityKey<?> key =
			 * getExceptionService().processException(exception, message,
			 * producer, severityType, ApplicationName.CCHT); cEx = new
			 * ControllerException(message, exception, key != null ? key
			 * .getKeyValueAsString() : null);
			 */
			cEx = new ControllerException(message, exception, null);
		}
		logger.error(cEx);
		return cEx;
	}

	public ControllerResult createControllerResult(ServiceOptimisticLockException optEx) {
		ControllerResult cResult = new ControllerResult();
		cResult.setCallStatus(optEx.getErrorType());
		if (optEx.getEntity() != null) {
			// AbstractKeyedEntity entity = optEx.getEntity();
			// TODO customize for the entity
		}
		return cResult;
	}

	public TermType getCronExpression(Schedule schedule) {
		if (schedule != null) {
			return new TermType(schedule.getCronExpression(), schedule.getFriendlyDescription());
		} else {
			return null;
		}
	}

	public static UserPrincipal getCurrentUser() {
		SecurityContext securityContext = SecurityContextHelper.getSecurityContext();
		return securityContext.getUserPrincipal();
	}

}
