package gov.va.med.ccht.ui.interceptor;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.httpclient.util.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

import gov.va.med.ccht.model.User;
import gov.va.med.ccht.ui.ajax.AjaxRequestHandler;
import gov.va.med.ccht.ui.common.UserNotifier;
import gov.va.med.ccht.ui.util.WebUtil;
import gov.va.med.fw.security.Permission;
import gov.va.med.fw.security.SecurityContextHelper;
import gov.va.med.fw.security.UserPrincipal;

@Component
public class CommonReferenceDataInterceptor implements HandlerInterceptor {
	private static final Logger log = LoggerFactory.getLogger(CommonReferenceDataInterceptor.class);

	public static final String ONE_TIME_USER_NOTIFICATION = "oneTimeUserNotification";

	@Value("#{cchtProperties['useMinifiedDependencies']}")
	private String useMinifiedDependencies;
	@Value("#{cchtProperties['session.pollingIntervalSeconds']}")
	private int sessionPollingIntervalSeconds;
	@Value("#{cchtProperties['session.failedRequestsCount']}")
	private int sessionFailedRequestsCount;
	@Value("#{cchtProperties['session.heartBeatTimeoutMillis']}")
	private int sessionHeartBeatTimeoutMillis;
	@Value("#{cchtProperties['session.idleAfterSeconds']}")
	private int sessionIdleAfterSeconds;
	
	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
			throws Exception {
		return true;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex)
			throws Exception {
	}

	protected boolean isAjax(HttpServletRequest request) {
		return AjaxRequestHandler.isAjax(request);
	}

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
			ModelAndView modelAndView) throws Exception {
		if (isAjax(request)) {
			log.debug("Request was an AJAX request, no need to populate reference data");
			return;
		}

		if (modelAndView == null) {
			log.debug("No modelAndView returned, the controller must have dealt with the response manually.");
			return;
		}

		Map<String, Object> model = modelAndView.getModel();
		if (model == null) {
			log.error(
					"No model in the ModelAndView, that is unexpected. Returning without adding common reference data.");
			return;
		}

		if (modelAndView.getView() instanceof RedirectView
				|| (modelAndView.isReference() && modelAndView.getViewName().startsWith("redirect:"))) {
			log.debug("View is null or a redirect, no need to populate reference data");
			return;
		}

		HttpSession session = request.getSession();

		session.setAttribute("sessionPollingIntervalSeconds", new Integer(sessionPollingIntervalSeconds));
		session.setAttribute("sessionFailedRequestsCount", new Integer(sessionFailedRequestsCount));
		session.setAttribute("sessionHeartBeatTimeoutMillis", new Integer(sessionHeartBeatTimeoutMillis));
		session.setAttribute("sessionIdleAfterSeconds", new Integer(sessionIdleAfterSeconds));
		
		String val = UserNotifier.getAndRemoveAttributeFromSession(session);
		if (val != null)
			model.put(ONE_TIME_USER_NOTIFICATION, val);

		/*
		 * Add all roles and permissions to model for standard use in
		 * <security:authorize> JSP tags - CPB
		 */
		WebUtil.addClassConstantsToModel(Permission.class, model, true);

		/*
		 * Add all date util constants as they are defined (without adding
		 * "DATE_UTIL_" before their names) - CPB
		 */
		WebUtil.addClassConstantsToModel(DateUtil.class, model, false);

		/* Add constants from WebUtils for global error display */
		WebUtil.addClassConstantsToModel(WebUtil.class, model);

		// waiting on Spring 4 - CPB
		// model.put("AJAX_CONTEXT_PATH_PREFIX",
		// CommonWebConfig.AJAX_CONTEXT_PATH_PREFIX);

		model.put("useMinifiedDependencies", useMinifiedDependencies);

		if (!model.containsKey("user"))
			model.put("user", getCurrentUser());
		if (!model.containsKey("userName"))
			model.put("userName", getCurrentUsername());
		if (!model.containsKey("userDisplayName"))
			model.put("userDisplayName", getCurrentUserDisplayName());

	}

	public static String getCurrentUserDisplayName() {
		User currentUser = getCurrentUser();
		if (currentUser == null)
			return "";

		StringBuilder userName = new StringBuilder();
		userName.append(currentUser.getFamilyName());
		userName.append(", ");
		userName.append(currentUser.getGivenName());

		String middleName = currentUser.getMiddleName();
		if (middleName != null && middleName.length() > 0) {
			userName.append(" ");
			userName.append(middleName);
		}

		return userName.toString();
	}

	public static User getCurrentUser() {
		UserPrincipal p = SecurityContextHelper.getSecurityContext().getUserPrincipal();
		if (p == null)
			return null;
		if (p instanceof User)
			return (User) p;
		throw new IllegalStateException("Unsupported type of user in security context: " + p.getClass());
	}

	public static String getCurrentUsername() {
		User currentUser = getCurrentUser();
		if (currentUser == null || currentUser.getUserCredentials() == null)
			return null;
		return currentUser.getUserCredentials().getUserID();
	}

}
